import logging

_LOGGER = logging.getLogger(__name__)

class LoxDimmer:
    """Class for node abstraction."""

    def __init__(self, id, name, device_type, room, cat, details={}):
        #_LOGGER.debug("{0} init".format(id))
        self._id = id
        self._name = name
        self._device_type = device_type
        self._room = room
        self._cat = cat
        self._details = details
        self._state = False
        self._brightness = 0
        self._color_temp = None
        self._hs_color = [0,0]
        self.count = 0
        self.async_callbacks = []

    #def register_device_updated_cb(self, device_updated_cb):
    #    """Register device updated callback."""
    #    self.device_updated_cbs.append(device_updated_cb)

    #def unregister_device_updated_cb(self, device_updated_cb):
    #    """Unregister device updated callback."""
    #    self.device_updated_cbs.remove(device_updated_cb)

    #async def after_update(self):
    #    """Execute callbacks after internal state has been changed."""
    #    for device_updated_cb in self.device_updated_cbs:
    #        # pylint: disable=not-callable
    #        await device_updated_cb(self)
    
            
    @property
    def id(self):
        return self._id

    @property
    def name(self):
        return self._room + " " + self._name

    @property
    def device_type(self):
        return self._device_type

    @property
    def room(self):
        #_LOGGER.debug("{0} room={1}".format(self._id, self._room))
        return self._room

    @property
    def category(self):
        return self._cat

    @property
    def details(self):
        return self._details

    @property
    def manufacturer_name(self):
        #_LOGGER.debug("{0} manufacturer_name={1}".format(self._id, 'Loxone'))
        return 'Loxone'    

    @property
    def state(self):
        return self._state

    @property
    def brightness(self):
        return self._brightness

    @property
    def color_temp(self):
        return self._color_temp

    @property
    def hs_color(self):
        return self._hs_color

    def register_async_callback(self, async_callback):
        #_LOGGER.debug("register_async_callback")
        self.async_callbacks.append(async_callback)

    def unregister_async_callback(self, callback):
        #_LOGGER.debug("unregister_async_callback")
        if callback in self.async_callbacks:
            self.async_callbacks.remove(callback)

    def async_update(self):

        for async_signal_update in self.async_callbacks:
            #_LOGGER.debug("  doing update callback")
            async_signal_update()

    def set_value(self, stateName, value):
        if self._device_type == "Switch" and stateName == "active":
            _LOGGER.debug("id:'{0}', name:'{1}', [Set Switch] - active={2}".format(self._id, self._name, value))
            if value == 1: 
                self._state = True
                self._brightness = 255
            else:
                self._state = False
                self._brightness = 0

            #_LOGGER.debug("Update Switch: {0}".format(device.name))
            self.async_update()

        elif self._device_type == "Dimmer" and stateName == "position":
            _LOGGER.debug("id:'{0}', name:'{1}', [Set Dimmer] - position={2}".format(self._id, self._name, value))
            self._brightness = value * 2.55
            if value > 0:
                self._state = True
            else:
                self._state = False

            #_LOGGER.debug("Update Dimmer: {0}".format(device.name))
            self.async_update()

        else:
            _LOGGER.debug("id:'{0}', name:'{1}', [ValueNotSet {2}] - {3}={4}".format(self._id, self._name, self._device_type, stateName, value))



"""Represent the client session."""

import asyncio
import logging
import datetime
import aiohttp
import binascii

_LOGGER = logging.getLogger(__name__)

STATE_STARTING = 'starting'
STATE_RUNNING = 'running'
STATE_STOPPED = 'stopped'
CONNECTING = 'connecting'

RETRY_TIMER = 20

class WSClient:
    """This class client."""

    def __init__(self, loop, host, port, username, password, async_session_callback, async_message_callback):
        """init"""
        #_LOGGER.debug("__init__")
        self.loop = loop
        self.session = None
        self.ws = None
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self._state = None
        self.async_session_handler_callback = async_session_callback
        self.async_message_handler_callback = async_message_callback

        _LOGGER.debug('  self.host: {0}'.format(self.host))
        _LOGGER.debug('  self.port: {0}'.format(self.port))

    @property
    def state(self):
        #_LOGGER.debug("state")
        """state"""
        return self._state

    @state.setter
    def state(self, value):
        """state"""
        #_LOGGER.debug("state.setter")
        self._state = value
        _LOGGER.debug('Set Websocket state: {0}'.format(value))
        self.async_session_handler_callback(self._state)

    def start(self):
        _LOGGER.debug("start")
        if self.state != STATE_RUNNING:
            self.state = STATE_STARTING
            self.loop.create_task(self.running())

    async def running(self):
        """Start websocket connection."""
        _LOGGER.debug("running")
        #url = "wss://{0}:{1}/ws/rfc6455".format(self.host, self.port)
        url = "ws://{0}:{1}/ws/rfc6455".format(self.host, self.port)
        #url = "ws://192.168.1.10:8080"
        _LOGGER.debug('  url: {0}'.format(url))
        try:
            self.session = aiohttp.ClientSession()
            self.ws = await self.session.ws_connect(url, protocols=('remotecontrol'))
            self.state = STATE_RUNNING

            async for msg in self.ws:
                if self.state == STATE_STOPPED:
                    break
                elif msg.type == aiohttp.WSMsgType.BINARY:
                    #_LOGGER.debug('Websocket BINARY data: {0}'.format(binascii.hexlify(msg.data)))
                    self.async_message_handler_callback(msg.data, True)
                elif msg.type == aiohttp.WSMsgType.TEXT:
                    #self._data = json.loads(msg.data)
                    #self.async_session_handler_callback('data')
                    #_LOGGER.debug('Websocket TEXT data: {0}'.format(msg.data))
                    self.async_message_handler_callback(msg.data, False)
                elif msg.type == aiohttp.WSMsgType.CLOSED:
                    _LOGGER.debug("CLOSED")
                    break
                elif msg.type == aiohttp.WSMsgType.ERROR:
                    _LOGGER.debug("ERROR")
                    break

        except aiohttp.ClientConnectorError:
            _LOGGER.debug("ClientConnectorError")
            if self.state != STATE_STOPPED:
                self.state = CONNECTING
                self.retry()
        except Exception as err:
            _LOGGER.error('Error {0}'.format(err))
            if self.state != STATE_STOPPED:
                self.state = CONNECTING
                self.retry()
        else:
            _LOGGER.debug("other websocket issue")
            if self.state != STATE_STOPPED:
                self.state = CONNECTING
                self.retry()

        _LOGGER.debug("Finished running")

    def retry(self):
        """Retry to connect."""
        self.loop.call_later(RETRY_TIMER, self.start)
        _LOGGER.debug('Reconnecting in %i.', RETRY_TIMER)

    def send(self, message):
        """send"""
        _LOGGER.debug("sending: {0}".format(message))
        if self.state == STATE_RUNNING:
            try:
                self.loop.create_task(self.ws.send_str(message))
            except Exception as err:
                _LOGGER.error('send Error {0}'.format(err))

    def stop(self):
        """Close websocket connection."""
        _LOGGER.debug("stop")
        self.state = STATE_STOPPED


import logging

_LOGGER = logging.getLogger(__name__)

class LoxLightControllerV2:
    """Class for LightControllerV2 abstraction."""

    def __init__(self, id, name, device_type, room, cat, details):
        #_LOGGER.debug("{0} init".format(id))
        self._id = id
        self._name = name
        self._device_type = device_type
        self._room = room
        self._cat = cat
        self._details = details
        self.async_callbacks = []

    @property
    def id(self):
        return self._id

    @property
    def name(self):
        return self._room + " " + self._name

    @property
    def device_type(self):
        return self._device_type

    @property
    def room(self):
        #_LOGGER.debug("{0} room={1}".format(self._id, self._room))
        return self._room

    @property
    def category(self):
        #_LOGGER.debug("{0} cat={1}".format(self._id, self._cat))
        return self._cat

    @property
    def details(self):
        return self._details


    @property
    def manufacturer_name(self):
        #_LOGGER.debug("{0} manufacturer_name={1}".format(self._id, 'Loxone'))
        return 'Loxone'    

    def register_async_callback(self, async_callback):
        #_LOGGER.debug("{0} register_async_callback".format(self._id))
        self.async_callbacks.append(async_callback)

    def unregister_async_callback(self, callback):
        #_LOGGER.debug("{0} unregister_async_callback".format(self._id))
        if callback in self.async_callbacks:
            self.async_callbacks.remove(callback)

    def async_update(self):
        for async_signal_update in self.async_callbacks:
            async_signal_update()


    def set_value(self, stateName, value):
        if self._device_type == "LightControllerV2" and stateName == "moodList":
            _LOGGER.debug("id:'{0}', name:'{1}', [SetValue {2}] - {3}={4}".format(self._id, self._name, self._device_type, stateName, value))
            #device.add_modes(value)

        else:
            _LOGGER.debug("id:'{0}', name:'{1}', [ValueNotSet {2}] - {3}={4}".format(self._id, self._name, self._device_type, stateName, value))



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="loxws",
    version="0.0.44",
    author="tjsmithuk",
    author_email="tsmith@clamfish.com",
    description="Loxone Client",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/tsmith/loxws",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'pycryptodome>=3.8.1',
        'aiohttp>=3.5.4'
    ]
)


import logging

_LOGGER = logging.getLogger(__name__)

class LoxGate:
    """Class for node abstraction."""

    def __init__(self, id, name, device_type, room, cat, details):
        self._id = id
        self._name = name
        self._device_type = device_type
        self._room = room
        self._cat = cat
        self._details = details
        self._position = False
        self.async_callbacks = []
            
    @property
    def id(self):
        return self._id

    @property
    def name(self):
        return self._room + " " + self._name

    @property
    def device_type(self):
        return self._device_type

    @property
    def room(self):
        return self._room

    @property
    def category(self):
        return self._cat

    @property
    def details(self):
        return self._details


    @property
    def manufacturer_name(self):
        return 'Loxone'    

    @property
    def position(self):
        return self._position

    def register_async_callback(self, async_callback):
        #_LOGGER.debug("register_async_callback")
        self.async_callbacks.append(async_callback)

    def unregister_async_callback(self, callback):
        #_LOGGER.debug("unregister_async_callback")
        if callback in self.async_callbacks:
            self.async_callbacks.remove(callback)

    #def async_set_position(self, position):
    #    _LOGGER.debug("{0} [{1}] async_set_position() position={2}".format(self._id, self._name, position))

    #def async_stop(self):
    #    _LOGGER.debug("{0} [{1}] async_stop()".format(self._id, self._name))

    def async_update(self):
        for async_signal_update in self.async_callbacks:
            #_LOGGER.debug("id:'{0}', name:'{1}', [async_update()] state={2}".format(self._id, self._name, self._position))
            async_signal_update()

    def set_value(self, stateName, value):
        if self._device_type == "Jalousie" and stateName == "position":
            _LOGGER.debug("id:'{0}', name:'{1}', [SetValue Gate] - state={2}".format(self._id, self._name, value))

            self._position = value

            self.async_update()

        else:
            _LOGGER.debug("id:'{0}', name:'{1}', [ValueNotSet {2}] - {3}={4}".format(self._id, self._name, self._device_type, stateName, value))
